<?php
/**
 * Plugin Name: EventsFrame Connector
 * Plugin URI: https://support.eventsframe.com/article/100-eventsframe-connector-wordpress-plugin
 * Description: Plugin which allows you to have EventsFrame events in your WordPress site.
 * Author: EventsFrame
 * Version: 1.04
 * Author URI: https://www.eventsframe.com
 * License: GPL2
 */

defined( 'ABSPATH' ) || exit;

if ( !class_exists( 'efrmPlugin' ) ) :

final class efrmPlugin {

  public $current_user = false;

  public $home_base = 'https://app.eventsframe.com';
  public $post_type = 'eventsframe';
  public $options_key = 'efrm-connector-options';
  public $transient_prefix = 'efrm_event_html_cache_';
  public $meta_id = 'efrm_event_id';

  public static function instance () {

    static $instance = null;

    if ( null === $instance ) {

      $instance = new efrmPlugin;
      $instance->setup_globals();
      $instance->includes();
      $instance->setup_actions();

      if ( is_admin() ) {

        efrm_build_admin();

      }

    }

    return $instance;

  }

  private function __construct () { /* No mans land &#9760; */  }
  public function __clone() {}
  public function __wakeup() {}

  private function setup_globals () {

    $this->file = __FILE__;
    $this->basename = plugin_basename( $this->file );
    $this->plugin_dir = plugin_dir_path( $this->file );
    $this->plugin_url = plugin_dir_url( $this->file );

    $this->includes_dir = trailingslashit( $this->plugin_dir . 'includes' );
    $this->includes_url = trailingslashit( $this->plugin_url . 'includes' );

  }

  private function includes () {

    if ( is_admin() ) {

      require( $this->includes_dir . 'admin/admin.php' );

    }

  }

  private function setup_actions () {

    register_activation_hook(   $this->file, array( $this, 'plugin_activation' ) );
    register_deactivation_hook( $this->file, array( $this, 'plugin_deactivation' ) );

    add_action( 'set_current_user', array( $this, 'set_current_user'      ) );
    add_action( 'init',             array( $this, 'reg_custom_post_type'  ) );

  }


  public function plugin_activation () {

    $this->reg_custom_post_type();
    $this->update_rewrite_rules();

  }

  public function plugin_deactivation () {

    $this->update_rewrite_rules();

  }

  private function update_rewrite_rules () {

    flush_rewrite_rules();

  }

  public function set_current_user () {

    $this->current_user = wp_get_current_user();

  }

  public function reg_custom_post_type () {

    $labels = array(
      'name'                => _x( 'EventsFrame Events', 'Post type general name', 'textdomain' ),
      'singular_name'       => _x( 'Event', 'Post type singular name', 'textdomain' ),
      'menu_name'           => _x( 'EventsFrame', 'Admin Menu text', 'textdomain' ),
      'name_admin_bar'      => _x( 'Event', 'Add New on Toolbar', 'textdomain' ),
      'add_new'             => __( 'Add New Event Page', 'textdomain' ),
      'add_new_item'        => __( 'Add New Event Page', 'textdomain' ),
      'new_item'            => __( 'New Event Page', 'textdomain' ),
      'edit_item'           => __( 'Edit Event Page', 'textdomain' ),
      'view_item'           => __( 'View Event', 'textdomain' ),
      'all_items'           => __( 'All Events', 'textdomain' ),
      'search_items'        => __( 'Search Events', 'textdomain' ),
      'not_found'           => __( 'No events found.', 'textdomain' ),
      'not_found_in_trash'  => __( 'No events found in Trash.', 'textdomain' ),
    );

    $args = array(
      'label'               => __( $this->post_type, 'text_domain' ),
      'description'         => __( 'EventsFrame Event', 'text_domain' ),
      'labels'              => $labels,
      'supports'            => array( ),
      'hierarchical'        => false,
      'rewrite'             => array( 'slug' => $this->post_type ),
      'menu_icon'           => 'dashicons-calendar',
      'public'              => true,
      'show_ui'             => true,
      'show_in_menu'        => true,
      'menu_position'       => 5,
      'show_in_admin_bar'   => true,
      'show_in_nav_menus'   => true,
      'can_export'          => false,
      'has_archive'         => false,
      'exclude_from_search' => true,
      'publicly_queryable'  => true,
      'capability_type'     => 'post',
    );

    register_post_type( $this->post_type, $args );
    remove_post_type_support( $this->post_type, 'editor' );

    add_filter( 'template_include', array( $this, 'efrm_template' ) );

  }

  private function load_event ( $event_id ) {

    $api_key = $this->get_option( 'api_key' );

    if ( !$api_key ) {

      return false;

    }

    list( $prfx1, $prfx2, $account_id, $token ) = explode( '_', $api_key );
    $url = sprintf( '%s/api/a/%s/wp/event/%s/', $this->home_base, $account_id, $event_id );

    $response = wp_remote_post( $url, array(
        'body' => array(
          'api_token' => $token,
        )
      )
    );

    if ( is_wp_error( $response ) ) {

      return false;

    }

    $data = json_decode( $response['body'] );

    if ( !is_object( $data ) || !property_exists( $data, 'success' ) || !$data->success ) {

      return false;

    }

    return $data->event;

  }

  public function efrm_template ( $template ) {

    if ( is_singular( array( $this->post_type) ) && !post_password_required() ) {

      $template = trailingslashit( $this->includes_dir . 'front' ) . 'template.php';

    }
    return $template;

  }

  private function render_404 () {

    status_header( 404 );
    nocache_headers();
    include( get_query_template( '404' ) );
    exit();

  }

  public function render_event ( $post_id ) {

      $event_id = get_post_meta( $post_id, $this->meta_id, true );

      $html = get_transient( $this->transient_prefix . $event_id );

      if ( empty( $html ) ) {

        $event = $this->load_event( $event_id );

        if ( $event == false ) {

          delete_transient( $this->transient_prefix . $event_id );
          $this->render_404();

        }

        $html = $event->html;
        set_transient( $this->transient_prefix . $event_id, $html, 60 * 60 * 24 ); // 24 hours

      }
      echo $html;
      exit();

  }

  public function set_default_options () {

    $options = array(
      'api_key' => null,
      'events'  => null,
    );

    update_option( $this->options_key, $options, true );

  }

  public function get_option ( $key ) {

    $options = get_option( $this->options_key );

    if ( $options && isset( $options[$key] ) ) {

      return $options[$key];

    }

    return false;

  }

  public function set_option ( $key, $value ) {

    $options = get_option( $this->options_key );

    $options[$key] = $value;

    update_option( $this->options_key, $options, true );

  }

  public function get_options () {

    return get_option( $this->options_key );

  }

  public function set_options ( $new_options ) {

    $old_options = get_option( $this->options_key );

    $options = array_merge( $old_options, $new_options );

    update_option( $this->options_key, $options, true );

  }

  public function is_authorized () {

    return ( is_admin() && $this->get_option( 'api_key' ) );

  }

  public function get_version () {

    return get_plugin_data( $this->file, false, false )['Version'];

  }

}

function efrmP () {

  return efrmPlugin::instance();

}

efrmP();

endif;