<?php defined( 'ABSPATH' ) || exit;

if ( !class_exists( 'efrmAdmin'  ) ) :

final class efrmAdmin {

  public function __construct () {

    $this->setup_globals();
    $this->setup_actions();

  }

  private function setup_globals () {

    $efrm = efrmP();
    $this->admin_dir = trailingslashit( $efrm->includes_dir . 'admin' );
    $this->admin_url = trailingslashit( $efrm->includes_url . 'admin' );

  }

  private function setup_actions () {

    add_action( 'admin_init',                      array( $this, 'add_meta_boxes' )             );
    add_action( 'admin_init',                      array( $this, 'register_settings' )          );
    add_action( 'admin_menu',                      array( $this, 'admin_menu' )                 );
    add_action( 'current_screen',                  array( $this, 'admin_screen' )               );
    add_action( 'admin_print_scripts',             array( $this, 'disable_autosave' )           );
    add_action( 'save_post_' . efrmP()->post_type, array( $this, 'save_post' )                  );

    add_filter( 'post_row_actions',                array( $this, 'remove_quick_edit' ),   10,  2);
    add_filter( 'wp_insert_post_data',             array( $this, 'save_post_data' )             );

  }

  public function remove_quick_edit( $actions, $post ) {

    if ( $post->post_type != efrmP()->post_type ) {

      return $actions;

    }
    unset( $actions['inline hide-if-no-js'] );
    return $actions;

  }

  public function hide_unused_publish_options () {

    echo '<style> div#submitdiv div#minor-publishing { display: none } </style>';
    echo '<style> div#submitdiv div#minor-publishing-actions { display: none } </style>';
    echo '<style> div#submitdiv div#misc-publishing-actions { display: none } </style>';
    echo '<style> div#submitdiv div#visibility { display: none } </style>';

  }

  public function admin_screen ($current_screen) {

    if ( efrmP()->post_type == $current_screen->post_type && 'post' == $current_screen->base ) {

        add_action( 'admin_head', array( $this, 'hide_unused_publish_options' ) );

    }

  }

  public function admin_menu () {

    add_options_page( 'EventsFrame Settings',
                      'EventsFrame',
                      'manage_options',      // Capability
                      'efrm_settings',       // Page
                      array( $this, 'plugin_settings' ));

  }

  function register_settings() {

    register_setting(
      'efrm_settings_group',                // Group
      efrmP()->options_key,                 // Option name
      array( $this, 'validate' )            // Sanitize
    );

    add_settings_section(
      'efrm_auth_settings',                 // Section ID
      'Auth Settings',                      // Title
      array( $this, 'print_section_info' ), // Callback
      'efrm_settings'                       // Page
    );

    add_settings_field(
      'api_key',                            // Field ID
      'Plugin API Key',                     // Field Title
      array( $this, 'api_key_callback' ),   // Callback
      'efrm_settings',                      // Page
      'efrm_auth_settings'                  // Section
    );

  }

  public function validate ( $input ) {

    $message = null;
    $type = null;

    $new_input = array();

    if ( isset( $input['api_key'] ) && strlen( $input['api_key'] ) != 0 ) {

      $api_key = sanitize_text_field( $input['api_key'] );

      if ( substr_count( $api_key , '_' ) != 3 ) {

        $type = 'error';
        $message = 'Invalid API Key format!';

      } else {

        list( $prfx1, $prfx2, $account_id, $token ) = explode( '_', $api_key );
        $url = sprintf( '%s/api/a/%s/wp/test/', efrmP()->home_base, $account_id );

        $response = wp_remote_post( $url, array(
            'body' => array(
              'api_token' => $token,
            )
          )
        );

        if ( is_wp_error( $response ) ) {

          $type = 'error';
          $message = 'Unable to validate API Key!';

        } else {

          $data = json_decode( $response['body'] );

          if ( !is_object( $data ) || !property_exists( $data, 'success' ) || !$data->success ) {

            $type = 'error';
            $message = 'Invalid API Key!';

          } else {

            $new_input['api_key'] = $api_key;

            $type = 'updated';
            $message = 'API Key validated and saved!';

          }

        }

      }

    } else {

      $type = 'error';
      $message = 'Missing API Key!';

    }

    add_settings_error(
        'api_key',
        esc_attr( 'settings_updated' ),
        $message,
        $type
    );

    return $new_input;
  }

  public function print_section_info () {

    print 'Enter your EventsFrame Plugin API Key below:';

  }

  public function api_key_callback () {

    printf(
      '<input type="text" id="api_key" name="%s[api_key]" value="%s" class="regular-text" placeholder="efrm_key_######_################" />',
      efrmP()->options_key,
      efrmP()->get_option( 'api_key' ) ? esc_attr( efrmP()->get_option( 'api_key' ) ) : ''
    );

  }

  public function plugin_settings () {

    if ( !current_user_can( 'manage_options'  ) )  {

      wp_die( 'You do not have sufficient permissions to access this page.' );

    }

    $reveal_key_url = efrmP()->home_base . '/wp-auth/?' . http_build_query( array( 'wp_site_url' => site_url() ) );

    include( $this->admin_dir . 'auth.php' );

  }

  public function add_meta_boxes () {

    add_meta_box( 'efrm-event-id', 'EventsFrame', array( $this, 'render_event_box' ), efrmP()->post_type, 'normal' );

    if ( !efrmP()->is_authorized() ) {

      remove_post_type_support( efrmp()->post_type, 'title' );
      remove_meta_box( 'submitdiv', efrmP()->post_type, 'side' );

    }

  }

  public function render_event_box ( $post ) {

    $events = $this->load_events();

    $event_id = get_post_meta( $post->ID, efrmP()->meta_id, true);

    include( $this->admin_dir . 'dropdown.php' );

  }

  private function load_events () {

    $stngs_url = menu_page_url( 'efrm_settings', false );

    $set_link = '<br><br>Please check the <a href="' . $stngs_url . '">plugin\'s settings</a>.';

    if ( !efrmP()->is_authorized() ) {

      wp_die( 'You are not authorized!' . $set_link );

    }

    $api_key = efrmP()->get_option( 'api_key' );

    if ( !$api_key ) {

      wp_die( 'Missing API Key!' . $set_link );

    }

    list( $prfx1, $prfx2, $account_id, $token ) = explode( '_', $api_key );
    $url = sprintf( '%s/api/a/%s/wp/events/', efrmP()->home_base, $account_id );

    $response = wp_remote_post( $url, array(
        'body' => array(
          'api_token' => $token,
        )
      )
    );

    if ( is_wp_error( $response ) ) {

      wp_die( 'API Error!' );

    }

    $data = json_decode( $response['body'] );

    if ( !is_object( $data ) || !property_exists( $data, 'success' ) || !$data->success ) {

      wp_die( 'API Call Failed!' );

    }

    return $data->events;

  }

  public function disable_autosave () {

    if ( get_post_type() == efrmP()->post_type ) {

      wp_deregister_script( 'autosave' );

    }

  }

  public function save_post_data ( $post ) {

    if ( 'trash' !== $post['post_status'] ) { // We still want to use the trash

      if ( $post['post_type'] == efrmP()->post_type ) {

        if ( $post['post_status'] == 'draft' ) {

          return; // Prevent saving draft!
        }

      }

    }
    return $post;

  }

  public function save_post ( $post_id ) {

    if ( isset( $_POST[efrmP()->meta_id] ) ) {

      $event_id = sanitize_text_field( $_POST[efrmP()->meta_id] );
      update_post_meta( $post_id, efrmP()->meta_id, $event_id );
      delete_transient( efrmP()->transient_prefix . $event_id );

    }

  }

}

endif;

function efrm_build_admin() {

  efrmP()->admin = new efrmAdmin();

}
